#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
    /// </summary>
    [Description("The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers")]
    public class MADH : Indicator
    {
        #region Variables
            private int shortLength 	= 8; // Default setting for ShortLength
            private int dominantCycle 	= 27; // Default setting for DominantCycle
			private double LongLength;
			private double Filt1;
			private double Filt2;
			private double coef;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Gold), PlotStyle.Line, "MADHPlot"));
            Add(new Line(Color.FromKnownColor(KnownColor.RoyalBlue), 0, "ZeroLine"));
            Overlay				= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            LongLength = Math.Truncate((double)ShortLength + (double)DominantCycle / 2);
			
			if (CurrentBar < LongLength)
				return;
			
			Filt1 = 0;
			coef = 0;
			
			for (int i = 1; i <= ShortLength; i++)
			{
				Filt1 += (1 - Math.Cos((Math.PI/180) * (360 * i/(ShortLength+1)))) * Close[i-1];
				coef += (1 - Math.Cos((Math.PI/180) * (360 * i/(ShortLength+1))));
			}
			
			if (coef != 0)
				Filt1 = Filt1 / coef;
			
			Filt2 = 0;
			coef = 0;
			
			for (int i = 1; i <= LongLength; i++)
			{
				Filt2 += (1 - Math.Cos((Math.PI/180) * (360 * i/(LongLength+1)))) * Close[i-1];
				coef += (1 - Math.Cos((Math.PI/180) * (360 * i/(LongLength+1))));
			}

			if (coef != 0)
				Filt2 = Filt2 / coef;
			
			if (Filt2 != 0)
				MADHPlot.Set(100 * (Filt1 - Filt2) / Filt2);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MADHPlot
        {
            get { return Values[0]; }
        }
		[Gui.Design.DisplayName ("Short Length")] 
        [Description("Short Length")]
        [GridCategory("Parameters")]
        public int ShortLength
        {
            get { return shortLength; }
            set { shortLength = Math.Max(1, value); }
        }
		[Gui.Design.DisplayName ("Dominant Cycle")] 
        [Description("Dominant Cycle")]
        [GridCategory("Parameters")]
        public int DominantCycle
        {
            get { return dominantCycle; }
            set { dominantCycle = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MADH[] cacheMADH = null;

        private static MADH checkMADH = new MADH();

        /// <summary>
        /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
        /// </summary>
        /// <returns></returns>
        public MADH MADH(int dominantCycle, int shortLength)
        {
            return MADH(Input, dominantCycle, shortLength);
        }

        /// <summary>
        /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
        /// </summary>
        /// <returns></returns>
        public MADH MADH(Data.IDataSeries input, int dominantCycle, int shortLength)
        {
            if (cacheMADH != null)
                for (int idx = 0; idx < cacheMADH.Length; idx++)
                    if (cacheMADH[idx].DominantCycle == dominantCycle && cacheMADH[idx].ShortLength == shortLength && cacheMADH[idx].EqualsInput(input))
                        return cacheMADH[idx];

            lock (checkMADH)
            {
                checkMADH.DominantCycle = dominantCycle;
                dominantCycle = checkMADH.DominantCycle;
                checkMADH.ShortLength = shortLength;
                shortLength = checkMADH.ShortLength;

                if (cacheMADH != null)
                    for (int idx = 0; idx < cacheMADH.Length; idx++)
                        if (cacheMADH[idx].DominantCycle == dominantCycle && cacheMADH[idx].ShortLength == shortLength && cacheMADH[idx].EqualsInput(input))
                            return cacheMADH[idx];

                MADH indicator = new MADH();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.DominantCycle = dominantCycle;
                indicator.ShortLength = shortLength;
                Indicators.Add(indicator);
                indicator.SetUp();

                MADH[] tmp = new MADH[cacheMADH == null ? 1 : cacheMADH.Length + 1];
                if (cacheMADH != null)
                    cacheMADH.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheMADH = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MADH MADH(int dominantCycle, int shortLength)
        {
            return _indicator.MADH(Input, dominantCycle, shortLength);
        }

        /// <summary>
        /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
        /// </summary>
        /// <returns></returns>
        public Indicator.MADH MADH(Data.IDataSeries input, int dominantCycle, int shortLength)
        {
            return _indicator.MADH(input, dominantCycle, shortLength);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MADH MADH(int dominantCycle, int shortLength)
        {
            return _indicator.MADH(Input, dominantCycle, shortLength);
        }

        /// <summary>
        /// The MADH (Moving Average Difference - Hann) indicator was published in the November 2021 Technical Analysis of Stocks and Commodities article titled 'The MAD Indicator, Enhanced' by John F. Ehlers
        /// </summary>
        /// <returns></returns>
        public Indicator.MADH MADH(Data.IDataSeries input, int dominantCycle, int shortLength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MADH(input, dominantCycle, shortLength);
        }
    }
}
#endregion
